package com.colinwyattwarren.tictactwice;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import android.util.Log;

public class Board {
	public enum Player {
		A, B
	}

	public enum SquareType {
		A, B, C, D, E, F, G, H, I, J, K, L, M, N, O, P
	}

	public class Point2D {
		public Point2D(int _x, int _y) {
			x = _x;
			y = _y;
		}

		public int x;
		public int y;
	}

	private static final SquareType[][] mFirstLayout = {
			{ SquareType.A, SquareType.B, SquareType.C, SquareType.D },
			{ SquareType.E, SquareType.F, SquareType.G, SquareType.H },
			{ SquareType.I, SquareType.J, SquareType.K, SquareType.L },
			{ SquareType.M, SquareType.N, SquareType.O, SquareType.P } };

	private static final SquareType[][] mSecondLayout = {
			{ SquareType.P, SquareType.O, SquareType.N, SquareType.M },
			{ SquareType.L, SquareType.K, SquareType.J, SquareType.I },
			{ SquareType.H, SquareType.G, SquareType.F, SquareType.E },
			{ SquareType.D, SquareType.C, SquareType.B, SquareType.A } };

	/*
	 * I know that the board could be stored more efficiently, or in a manner
	 * which makes it easier/faster to check for a winner. However, this layout
	 * greatly simplifies both the board code and the graphics code. Given how
	 * little the check for winner function is called, I'm not going to worry
	 * about it unless there is noticeable lag when adding a piece.
	 */
	private SquareType[][] mLayout;
	private List<Point2D> mPlayerAPieces;
	private List<Point2D> mPlayerBPieces;
	private Map<SquareType, Point2D> mReverseLookupTable;

	public Board(boolean first) {
		if (first) {
			mLayout = mFirstLayout;
		} else {
			mLayout = mSecondLayout;
		}

		mPlayerAPieces = new LinkedList<Board.Point2D>();
		mPlayerBPieces = new LinkedList<Board.Point2D>();

		buildReverseLookupTable();
	}

	private void buildReverseLookupTable() {
		mReverseLookupTable = new HashMap<Board.SquareType, Board.Point2D>(16);

		for (int x = 0; x < 4; x++) {
			for (int y = 0; y < 4; y++) {
				mReverseLookupTable.put(mLayout[x][y], new Point2D(x, y));
			}
		}
	}

	public void placePiece(Player player, int x, int y) {
		switch (player) {
		case A:
			mPlayerAPieces.add(new Point2D(x, y));
			break;
		case B:
			mPlayerBPieces.add(new Point2D(x, y));
			break;
		default:
			Log.w(Board.class.getName(), "Invalid player passed to placePiece");
		}
	}

	public void placePiece(Player player, SquareType squareType) {
		Point2D loc = mReverseLookupTable.get(squareType);
		placePiece(player, loc.x, loc.y);
	}

	public SquareType[][] getLayout() {
		return mLayout;
	}

	public List<Point2D> getPlayerPositions(Player player) {
		switch (player) {
		case A:
			return mPlayerAPieces;
		case B:
			return mPlayerBPieces;
		default:
			Log.w(Board.class.getName(), "Invalid player passed to getPlayerPositions");
			return new LinkedList<Point2D>();
		}
	}
	
	public boolean squareHasPiece(SquareType square) {
		Point2D loc = mReverseLookupTable.get(square);
		
		for (Point2D point : mPlayerAPieces) {
			if (loc.x == point.x && loc.y == point.y)
				return true;
		}
		
		for (Point2D point : mPlayerBPieces) {
			if (loc.x == point.x && loc.y == point.y)
				return true;
		}
		
		return false;
	}
	
	public Player getWinner() {
		if (checkPlayer(Player.A))
			return Player.A;
		if (checkPlayer(Player.B))
			return Player.B;
		return null;
	}
	
	private boolean checkPlayer(Player player) {
		boolean[][] checkTable = new boolean[4][4];
		
		List<Point2D> points;
		switch(player) {
		case A:
			points = mPlayerAPieces;
			break;
		case B:
			points = mPlayerBPieces;
			break;
		default: // An invalid player can't win!
			Log.w(Board.class.getName(), "Invalid player passed to checkPlayer");
			return false;
		}
		
		for (Point2D point : points) {
			checkTable[point.x][point.y] = true;
		}
		
		// Check
		return (// Vertical
				(checkTable[0][0] && checkTable[0][1] && checkTable[0][2] && checkTable[0][3])
				|| (checkTable[1][0] && checkTable[1][1] && checkTable[1][2] && checkTable[1][3])
				|| (checkTable[2][0] && checkTable[2][1] && checkTable[2][2] && checkTable[2][3])
				|| (checkTable[3][0] && checkTable[3][1] && checkTable[3][2] && checkTable[3][3])
				
				// Horizontal
				|| (checkTable[0][0] && checkTable[1][0] && checkTable[2][0] && checkTable[3][0])
				|| (checkTable[0][1] && checkTable[1][1] && checkTable[2][1] && checkTable[3][1])
				|| (checkTable[0][2] && checkTable[1][2] && checkTable[2][2] && checkTable[3][2])
				|| (checkTable[0][3] && checkTable[1][3] && checkTable[2][3] && checkTable[3][3])
				
				// Diagonal
				|| (checkTable[0][0] && checkTable[1][1]  && checkTable[2][2] && checkTable[3][3])
				|| (checkTable[0][3] && checkTable[1][2]  && checkTable[2][1] && checkTable[3][0]));
	}

	public SquareType getSquareForCoordinates(int pieceX, int pieceY) {
		return mLayout[pieceX][pieceY];
	}
}
