package com.colinwyattwarren.tictactwice;

import java.util.List;

import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

public class GameBoard extends View {
	// Static
	public static final int TITLE_POSITION_TOP = 0;
	public static final int TITLE_POSITION_BOTTOM = 1;
	public static final int TITLE_POSITION_LEFT = 2;
	public static final int TITLE_POSITION_RIGHT = 3;

	// Settings
	private boolean mShowTitle;
	private String mTitle;
	private int mTitlePosition;

	// Data
	private Board mBoard = null;
	private GameController mGameController = null;

	// Drawing
	private Paint mTitlePaint;
	private Paint mPiecePaint;
	private Paint mSquarePaints[];

	private int mTitleHeight = 50;
	private int mTitleWidth;

	private int mTitleX;
	private int mTitleY;

	private float mStartX;
	private float mStartY;

	private float mHeight;
	private float mWidth;

	private float mPieceHeight;
	private float mPieceWidth;

	int mSquareHeight;
	int mSquareWidth;

	Board.SquareType[][] mSquares;
	List<Board.Point2D> mPieces;

	public GameBoard(Context context, AttributeSet attrs) {
		super(context, attrs);

		TypedArray a = context.getTheme().obtainStyledAttributes(attrs,
				R.styleable.GameBoard, 0, 0);

		try {
			mShowTitle = a.getBoolean(R.styleable.GameBoard_showTitle, false);
			mTitle = a.getString(R.styleable.GameBoard_title);
			setTitlePosition(a.getInt(R.styleable.GameBoard_titlePosition, 0));
		} finally {
			a.recycle();
		}

		init();
	}

	private void init() {
		// Title
		mTitlePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		mTitlePaint.setColor(Color.BLACK);
		if (mTitleHeight == 0) {
			mTitleHeight = (int) mTitlePaint.getTextSize();
		} else {
			mTitlePaint.setTextSize(mTitleHeight);
		}

		// Board Lines
		mPiecePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
		mPiecePaint.setColor(Color.WHITE);
		mPiecePaint.setStyle(Paint.Style.STROKE);
		mPiecePaint.setStrokeWidth(10);

		// Board square paints
		mSquarePaints = new Paint[16];

		for (int i = 0; i < 16; i++) {
			// enjoy :D
			int color = (0xFF << 24) | (((i % 4) * (0xFF / 3)) << 16)
					| (((i / 4) * (0xFF / 3)) << 8)
					| (((i % 4) ^ (i / 4)) * (0xFF / 3));
			mSquarePaints[i] = new Paint(Paint.ANTI_ALIAS_FLAG);
			mSquarePaints[i].setColor(color);
			mSquarePaints[i].setStyle(Paint.Style.FILL);
		}

		// TODO: Do board stuff
		/*
		 * mBoard = new Board(false); mBoard.placePiece(Board.Player.A,
		 * SquareType.F); mBoard.placePiece(Board.Player.B, SquareType.C);
		 */
	}

	/**
	 * Calculates dimensions whenever the size is changed.
	 * 
	 * @param w
	 *            New width
	 * @param h
	 *            New height
	 * @param oldw
	 *            Old width
	 * @param oldh
	 *            Old height
	 */
	@Override
	public void onSizeChanged(int w, int h, int oldw, int oldh) {
		// Padding
		float xpad = (float) (getPaddingLeft() + getPaddingRight());
		float ypad = (float) (getPaddingTop() + getPaddingBottom());

		mStartX = 0;
		mStartY = 0;

		// Title
		if (mShowTitle) {
			if (mTitlePosition == TITLE_POSITION_TOP) { // Pad the top
				mStartY += mTitleHeight;
				ypad += mTitleHeight;
			} else if (mTitlePosition == TITLE_POSITION_BOTTOM) { // Pad the
																	// bottom
				ypad += mTitleHeight;
			} else if (mTitlePosition == TITLE_POSITION_LEFT) { // Pad the left
				mStartX += mTitleWidth;
				xpad += mTitleWidth;
			} else { // Pad the right
				xpad += mTitleWidth;
			}
		}

		// Calculate
		mWidth = (float) w - xpad;
		mHeight = (float) h - ypad;

		mSquareWidth = (int) mWidth / 4;
		mSquareHeight = (int) mHeight / 4;

		mPieceWidth = (float) (mSquareWidth * 0.75);
		mPieceHeight = (float) (mSquareHeight * 0.75);
	}

	/**
	 * Draws the board and title on the screen.
	 * 
	 * @param canvas
	 *            The canvas to draw on
	 */
	@Override
	public void onDraw(Canvas canvas) {
		super.onDraw(canvas);

		if (mBoard == null) {
			Log.w(GameBoard.class.getName(), "Board null in onDraw!");
			return;
		}

		// Draw background
		canvas.drawColor(Color.MAGENTA);

		// Draw Title
		if (mShowTitle) {
			canvas.drawText(mTitle, mTitleX, mTitleY, mTitlePaint);
		}

		// Draw Lines
		// TODO: Actually draw lines

		// Draw Squares
		mSquares = mBoard.getLayout();

		for (int i = 0; i < 16; i++) {
			int x = (int) (mStartX + (int) (mWidth / 4) * (i % 4));
			int y = (int) (mStartY + (int) (mHeight / 4) * (i / 4));
			canvas.drawRect(x, y, x + mSquareWidth, y + mSquareHeight,
					mSquarePaints[mSquares[i % 4][i / 4].ordinal()]);
		}

		// Draw Pieces
		for (int i = 0; i < 2; i++) {
			mPieces = mBoard.getPlayerPositions(Board.Player.values()[i]);

			for (Board.Point2D piece : mPieces) {
				int centerX = (int) (mStartX + (int) (mWidth / 4) * piece.x + mSquareWidth / 2);
				int centerY = (int) (mStartY + (int) (mHeight / 4) * piece.y + mSquareHeight / 2);
				int x = (int) (centerX - mPieceWidth / 2);
				int y = (int) (centerY - mPieceHeight / 2);

				switch (Board.Player.values()[i]) {
				case A: // X for player A
					canvas.drawLine(x, y, x + mPieceWidth, y + mPieceHeight,
							mPiecePaint);
					canvas.drawLine(x + mPieceWidth, y, x, y + mPieceHeight,
							mPiecePaint);
					break;
				case B: // O for player B
					canvas.drawCircle(centerX, centerY, centerX - x,
							mPiecePaint);
					break;
				}
			}
		}
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		float x = event.getX() - mStartX;
		float y = event.getY() - mStartY;
		
		int pieceX = (int) (x / (mWidth / 4));
		int pieceY = (int) (y / (mHeight / 4));
		
		Log.d(GameBoard.class.getName(), String.format("Coords: (%f, %f)", x, y));
		Log.d(GameBoard.class.getName(), String.format("Piece: (%d, %d)", pieceX, pieceY));
		
		if (x <= mWidth && y <= mHeight) {
			Board.SquareType square = mBoard.getSquareForCoordinates(pieceX, pieceY);
			mGameController.placePiece(square);
		}
		
		return super.onTouchEvent(event);	
	}

	public boolean getShowTitle() {
		return mShowTitle;
	}

	public void setShowTitle(boolean showTitle) {
		mShowTitle = showTitle;

		invalidate();
		requestLayout();
	}

	public String getTitle() {
		return mTitle;
	}

	public void setTitle(String title) {
		mTitle = title;

		// If the title is shown, redraw, if it isn't, we don't need to
		if (mShowTitle) {
			invalidate();
			requestLayout();
		}
	}

	public int getTitlePosition() {
		return mTitlePosition;
	}

	public void setTitlePosition(int titlePosition) {
		mTitlePosition = titlePosition;

		// If the title is shown, redraw, if it isn't, we don't need to
		if (mShowTitle) {
			invalidate();
			requestLayout();
		}
	}

	/**
	 * Sets the board to display.
	 * 
	 * @param board
	 *            The board to display
	 */
	public void setBoard(Board board) {
		mBoard = board;
	}

	/**
	 * Invalidates the state of the board, so the new state will be drawn.
	 */
	public void boardChanged() {
		invalidate();
	}

	public void setGameController(GameController gameController) {
		mGameController = gameController;
	}

}
